@@ -13,6 +13,7 @@ def api_v4(self):
 
     # The API commands for /zones/
     zones(self)
+    zones_access(self)
     zones_amp(self)
     zones_analytics(self)
     zones_argo(self)
@@ -29,6 +30,7 @@ def api_v4(self):
     zones_settings(self)
     zones_spectrum(self)
     zones_ssl(self)
+    zones_waiting_rooms(self)
     zones_workers(self)
 
     # The API commands for /railguns/
@@ -42,8 +44,10 @@ def api_v4(self):
 
     # The API commands for /accounts/
     accounts(self)
+    accounts_access(self)
     accounts_addressing(self)
     accounts_audit_logs(self)
+    accounts_diagnostics(self)
     accounts_firewall(self)
     accounts_load_balancers(self)
     accounts_secondary_dns(self)
@@ -76,11 +80,6 @@ def zones(self):
     """ API core commands for Cloudflare API"""
 
     self.add('AUTH', "zones")
-    self.add('VOID', "zones", "access")
-    self.add('AUTH', "zones", "access/apps")
-    self.add('AUTH', "zones", "access/apps/policies")
-    self.add('AUTH', "zones", "access/apps", "revoke-tokens")
-    self.add('AUTH', "zones", "access/certificates")
     self.add('AUTH', "zones", "activation_check")
     self.add('AUTH', "zones", "available_plans")
     self.add('AUTH', "zones", "available_rate_plans")
@@ -194,7 +193,7 @@ def zones_amp(self):
     """ API core commands for Cloudflare API"""
 
     self.add('VOID', "zones", "amp")
-    self.add('AUTH', "zones", "amp/viewer")
+    self.add('AUTH', "zones", "amp/sxg")
 
 def zones_logpush(self):
     """ API core commands for Cloudflare API"""
@@ -308,10 +307,12 @@ def user_load_balancers(self):
     self.add('VOID', "user/load_balancers")
     self.add('AUTH', "user/load_balancers/monitors")
     self.add('AUTH', "user/load_balancers/monitors", "preview")
+    self.add('AUTH', 'user/load_balancers/monitors', 'references')
     self.add('AUTH', "user/load_balancers/preview")
     self.add('AUTH', "user/load_balancers/pools")
     self.add('AUTH', "user/load_balancers/pools", "health")
     self.add('AUTH', "user/load_balancers/pools", "preview")
+    self.add('AUTH', 'user/load_balancers/pools', 'references')
 
 def user_workers(self):
     """ API core commands for Cloudflare API"""
@@ -327,9 +328,8 @@ def user_audit_logs(self):
 def user_load_balancing_analytics(self):
     """ API core commands for Cloudflare API"""
 
-    self.add('VOID', "user", "load_balancing_analytics")
-    self.add('AUTH', "user", "load_balancing_analytics/events")
-    self.add('AUTH', "user", "load_balancing_analytics/entities")
+    self.add('VOID', "user/load_balancing_analytics")
+    self.add('AUTH', "user/load_balancing_analytics/events")
 
 def user_tokens_verify(self):
     """ API core commands for Cloudflare API"""
@@ -343,25 +343,12 @@ def accounts(self):
     """ API core commands for Cloudflare API"""
 
     self.add('AUTH', "accounts")
-    self.add('VOID', "accounts", "access")
-    self.add('AUTH', 'accounts', 'access/certificates')
-    self.add('AUTH', "accounts", "access/groups")
-    self.add('AUTH', "accounts", "access/identity_providers")
-    self.add('AUTH', "accounts", "access/organizations")
-    self.add('AUTH', "accounts", "access/organizations/revoke_user")
-    self.add('AUTH', "accounts", "access/service_tokens")
-    self.add('VOID', "accounts", "access/logs")
-    self.add('AUTH', 'accounts', 'access/logs/access_requests')
-    self.add('AUTH', 'accounts', 'access/apps')
-    self.add('AUTH', 'accounts', 'access/apps', 'ca')
-    self.add('AUTH', 'accounts', 'access/apps', 'policies')
-    self.add('AUTH', 'accounts', 'access/apps', 'revoke-tokens')
     self.add('VOID', "accounts", "billing")
     self.add('AUTH', "accounts", "billing/profile")
     self.add('AUTH', "accounts", "custom_pages")
     self.add('AUTH', "accounts", "members")
     self.add('AUTH', "accounts", "railguns")
-    self.add('AUTH', "accounts", "railguns/connections")
+    self.add('AUTH', "accounts", "railguns", "connections")
     self.add('VOID', "accounts", "registrar")
     self.add('AUTH', "accounts", "registrar/domains")
     self.add('AUTH', "accounts", "roles")
@@ -369,6 +356,9 @@ def accounts(self):
     self.add('AUTH', 'accounts', 'rules/lists')
     self.add('AUTH', 'accounts', 'rules/lists', 'items')
     self.add('AUTH', 'accounts', 'rules/lists/bulk_operations')
+    self.add('AUTH', 'accounts', 'rulesets')
+    self.add('AUTH', 'accounts', 'rulesets', 'versions')
+    self.add('AUTH', 'accounts', 'rulesets/import')
     self.add('VOID', "accounts", "storage")
     self.add('AUTH', "accounts", "storage/analytics")
     self.add('AUTH', "accounts", "storage/analytics/stored")
@@ -378,12 +368,15 @@ def accounts(self):
     self.add('AUTH', "accounts", "storage/kv/namespaces", "keys")
     self.add('AUTH', "accounts", "storage/kv/namespaces", "values")
     self.add('AUTH', "accounts", "subscriptions")
+    self.add('AUTH', 'accounts', 'tunnels')
+    self.add('AUTH', 'accounts', 'tunnels', 'connections')
     self.add('AUTH', "accounts", "virtual_dns")
-    self.add('VOID', "accounts", "virtual_dns/dns_analytics")
-    self.add('AUTH', "accounts", "virtual_dns/dns_analytics/report")
-    self.add('AUTH', "accounts", "virtual_dns/dns_analytics/report/bytime")
+    self.add('VOID', "accounts", "virtual_dns", "dns_analytics")
+    self.add('AUTH', "accounts", "virtual_dns", "dns_analytics/report")
+    self.add('AUTH', "accounts", "virtual_dns", "dns_analytics/report/bytime")
     self.add('VOID', "accounts", "workers")
     self.add('AUTH', "accounts", "workers/scripts")
+    self.add('AUTH', 'accounts', 'workers/scripts', 'schedules')
 
 def accounts_addressing(self):
     """ API core commands for Cloudflare API"""
@@ -406,9 +399,12 @@ def accounts_load_balancers(self):
     self.add('AUTH', 'accounts', 'load_balancers/preview')
     self.add('AUTH', "accounts", "load_balancers/monitors")
     self.add('AUTH', 'accounts', 'load_balancers/monitors', 'preview')
+    self.add('AUTH', 'accounts', 'load_balancers/monitors', 'references')
     self.add('AUTH', "accounts", "load_balancers/pools")
     self.add('AUTH', "accounts", "load_balancers/pools", "health")
     self.add('AUTH', 'accounts', 'load_balancers/pools', 'preview')
+    self.add('AUTH', 'accounts', 'load_balancers/pools', 'references')
+    self.add('AUTH', 'accounts', 'load_balancers/regions')
     self.add('AUTH', 'accounts', 'load_balancers/search')
 
 def accounts_firewall(self):
@@ -423,18 +419,21 @@ def accounts_secondary_dns(self):
 
     self.add('VOID', "accounts", "secondary_dns")
     self.add('AUTH', "accounts", "secondary_dns/masters")
+    self.add('AUTH', 'accounts', 'secondary_dns/primaries')
     self.add('AUTH', "accounts", "secondary_dns/tsigs")
 
 def accounts_stream(self):
     """ API core commands for Cloudflare API"""
 
     self.add('AUTH', "accounts", "stream")
+    self.add('AUTH', "accounts", "stream", "captions")
     self.add('AUTH', "accounts", "stream/copy")
     self.add('AUTH', "accounts", "stream/direct_upload")
-    self.add('AUTH', "accounts", "stream/embed")
+    self.add('AUTH', "accounts", "stream", "embed")
     self.add('AUTH', "accounts", "stream/keys")
     self.add('AUTH', "accounts", "stream/preview")
-    self.add('AUTH', "accounts", "stream", "captions")
+    self.add('AUTH', "accounts", "stream/watermarks")
+    self.add('AUTH', "accounts", "stream/webhook")
 
 def zones_media(self):
     """ API core commands for Cloudflare API"""
@@ -452,3 +451,50 @@ def graphql(self):
     """ API core commands for Cloudflare API"""
 
     self.add('AUTH', "graphql")
+
+def zones_access(self):
+    """ API core commands for Cloudflare API"""
+
+    self.add('VOID', "zones", "access")
+    self.add('AUTH', "zones", "access/apps")
+    self.add('AUTH', "zones", "access/apps", "policies")
+    self.add('AUTH', "zones", "access/apps", "revoke_tokens")
+    self.add('AUTH', "zones", "access/certificates")
+    #self.add('AUTH', "zones", "access/apps/ca")
+    self.add('AUTH', "zones", "access/apps", "ca")
+    self.add('AUTH', "zones", "access/groups")
+    self.add('AUTH', "zones", "access/identity_providers")
+    self.add('AUTH', "zones", "access/organizations")
+    self.add('AUTH', "zones", "access/organizations/revoke_user")
+    self.add('AUTH', "zones", "access/service_tokens")
+
+def accounts_access(self):
+    """ API core commands for Cloudflare API"""
+
+    self.add('VOID', "accounts", "access")
+    self.add('AUTH', 'accounts', 'access/certificates')
+    self.add('AUTH', "accounts", "access/groups")
+    self.add('AUTH', "accounts", "access/identity_providers")
+    self.add('AUTH', "accounts", "access/organizations")
+    self.add('AUTH', "accounts", "access/organizations/revoke_user")
+    self.add('AUTH', "accounts", "access/service_tokens")
+    self.add('VOID', "accounts", "access/logs")
+    self.add('AUTH', 'accounts', 'access/logs/access_requests')
+    self.add('AUTH', 'accounts', 'access/apps')
+    #self.add('AUTH', 'accounts', 'access/apps/ca')
+    self.add('AUTH', 'accounts', 'access/apps', 'ca')
+    self.add('AUTH', 'accounts', 'access/apps', 'policies')
+    self.add('AUTH', 'accounts', 'access/apps', 'revoke_tokens')
+
+def accounts_diagnostics(self):
+    """ API core commands for Cloudflare API"""
+
+    self.add('VOID', 'accounts', 'diagnostics')
+    self.add('AUTH', 'accounts', 'diagnostics/traceroute')
+
+def zones_waiting_rooms(self):
+    """ API core commands for Cloudflare API"""
+
+    self.add('AUTH', 'zones', 'waiting_rooms')
+    self.add('AUTH', 'zones', 'waiting_rooms', 'status')
+    self.add('AUTH', 'zones', 'waiting_rooms/preview')
