@@ -14,16 +14,26 @@ try:
 except ImportError:
     jsonlines = None
 
+import CloudFlare
 from . import converters
 
-import CloudFlare
-
-def dump_commands():
+def dump_commands(cf):
     """dump a tree of all the known API commands"""
-    cf = CloudFlare.CloudFlare()
     w = cf.api_list()
     sys.stdout.write('\n'.join(w) + '\n')
 
+def dump_commands_from_web(cf):
+    """dump a tree of all the known API commands - from web"""
+    w = cf.api_from_web()
+    for r in w:
+        if r['deprecated']:
+            if r['deprecated_already']:
+                sys.stdout.write('%-6s %s ; deprecated %s - expired!\n' % (r['action'], r['cmd'], r['deprecated_date']))
+            else:
+                sys.stdout.write('%-6s %s ; deprecated %s\n' % (r['action'], r['cmd'], r['deprecated_date']))
+        else:
+            sys.stdout.write('%-6s %s\n' % (r['action'], r['cmd']))
+
 def run_command(cf, method, command, params=None, content=None, files=None):
     """run the command line"""
     # remove leading and trailing /'s
@@ -44,6 +54,7 @@ def run_command(cf, method, command, params=None, content=None, files=None):
 
     hex_only = re.compile('^[0-9a-fA-F]+$')
     waf_rules = re.compile('^[0-9]+[A-Z]*$')
+    uuid_value = re.compile('^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$')	# 8-4-4-4-12
 
     m = cf
     for element in parts:
@@ -53,40 +64,62 @@ def run_command(cf, method, command, params=None, content=None, files=None):
                 if len(element) in [32, 40, 48] and hex_only.match(element):
                     # raw identifier - lets just use it as-is
                     identifier1 = element
+                elif len(element) == 36  and uuid_value.match(element):
+                    # uuid identifier - lets just use it as-is
+                    identifier1 = element
                 elif element[0] == ':':
                     # raw string - used for workers script_name - use ::script_name
                     identifier1 = element[1:]
-                elif cmd[0] == 'certificates':
-                    # identifier1 = convert_certificates_to_identifier(cf, element)
-                    identifier1 = converters.convert_zones_to_identifier(cf, element)
-                elif cmd[0] == 'zones':
-                    identifier1 = converters.convert_zones_to_identifier(cf, element)
-                elif cmd[0] == 'organizations':
-                    identifier1 = converters.convert_organizations_to_identifier(cf, element)
-                elif (cmd[0] == 'user') and (cmd[1] == 'organizations'):
-                    identifier1 = converters.convert_organizations_to_identifier(cf, element)
-                elif (cmd[0] == 'user') and (cmd[1] == 'invites'):
-                    identifier1 = converters.convert_invites_to_identifier(cf, element)
-                elif (cmd[0] == 'user') and (cmd[1] == 'virtual_dns'):
-                    identifier1 = converters.convert_virtual_dns_to_identifier(cf, element)
-                elif (cmd[0] == 'user') and (cmd[1] == 'load_balancers') and (cmd[2] == 'pools'):
-                    identifier1 = converters.convert_load_balancers_pool_to_identifier(cf, element)
                 else:
-                    exit("/%s/%s :NOT CODED YET 1" % ('/'.join(cmd), element))
+                    try:
+                        if cmd[0] == 'certificates':
+                            # identifier1 = convert_certificates_to_identifier(cf, element)
+                            identifier1 = converters.convert_zones_to_identifier(cf, element)
+                        elif cmd[0] == 'zones':
+                            identifier1 = converters.convert_zones_to_identifier(cf, element)
+                        elif cmd[0] == 'accounts':
+                            identifier1 = converters.convert_accounts_to_identifier(cf, element)
+                        elif cmd[0] == 'organizations':
+                            identifier1 = converters.convert_organizations_to_identifier(cf, element)
+                        elif (cmd[0] == 'user') and (cmd[1] == 'organizations'):
+                            identifier1 = converters.convert_organizations_to_identifier(cf, element)
+                        elif (cmd[0] == 'user') and (cmd[1] == 'invites'):
+                            identifier1 = converters.convert_invites_to_identifier(cf, element)
+                        elif (cmd[0] == 'user') and (cmd[1] == 'virtual_dns'):
+                            identifier1 = converters.convert_virtual_dns_to_identifier(cf, element)
+                        elif (cmd[0] == 'user') and (cmd[1] == 'load_balancers') and (cmd[2] == 'pools'):
+                            identifier1 = converters.convert_load_balancers_pool_to_identifier(cf, element)
+                        else:
+                            raise Exception("/%s/%s :NOT CODED YET" % ('/'.join(cmd), element))
+                    except Exception as e:
+                        sys.stderr.write('cli4: /%s - %s\n' % (command, e))
+                        raise e
                 cmd.append(':' + identifier1)
             elif identifier2 is None:
                 if len(element) in [32, 40, 48] and hex_only.match(element):
                     # raw identifier - lets just use it as-is
                     identifier2 = element
+                elif len(element) == 36  and uuid_value.match(element):
+                    # uuid identifier - lets just use it as-is
+                    identifier2 = element
                 elif element[0] == ':':
                     # raw string - used for workers script_names
                     identifier2 = element[1:]
-                elif (cmd[0] and cmd[0] == 'zones') and (cmd[2] and cmd[2] == 'dns_records'):
-                    identifier2 = converters.convert_dns_record_to_identifier(cf,
-                                                                              identifier1,
-                                                                              element)
                 else:
-                    exit("/%s/%s :NOT CODED YET 2" % ('/'.join(cmd), element))
+                    try:
+                        if (cmd[0] and cmd[0] == 'zones') and (cmd[2] and cmd[2] == 'dns_records'):
+                            identifier2 = converters.convert_dns_record_to_identifier(cf,
+                                                                                      identifier1,
+                                                                                      element)
+                        elif (cmd[0] and cmd[0] == 'zones') and (cmd[2] and cmd[2] == 'custom_hostnames'):
+                            identifier2 = converters.convert_custom_hostnames_to_identifier(cf,
+                                                                                      identifier1,
+                                                                                      element)
+                        else:
+                            raise Exception("/%s/%s :NOT CODED YET" % ('/'.join(cmd), element))
+                    except Exception as e:
+                        sys.stderr.write('cli4: /%s - %s\n' % (command, e))
+                        raise e
                 # identifier2 may be an array - this needs to be dealt with later
                 if isinstance(identifier2, list):
                     cmd.append(':' + '[' + ','.join(identifier2) + ']')
@@ -97,23 +130,39 @@ def run_command(cf, method, command, params=None, content=None, files=None):
                 if len(element) in [32, 40, 48] and hex_only.match(element):
                     # raw identifier - lets just use it as-is
                     identifier3 = element
+                elif len(element) == 36  and uuid_value.match(element):
+                    # uuid identifier - lets just use it as-is
+                    identifier3 = element
                 elif waf_rules.match(element):
                     identifier3 = element
+                elif element[0] == ':':
+                    # raw string - used for workers script_names
+                    identifier3 = element[1:]
                 else:
-                    exit("/%s/%s :NOT CODED YET 3" % ('/'.join(cmd), element))
+                    if len(cmd) >= 6 and cmd[0] == 'accounts' and cmd[2] == 'storage' and cmd[3] == 'kv' and cmd[4] == 'namespaces' and cmd[6] == 'values':
+                        identifier3 = element
+                    else:
+                        sys.stderr.write('/%s/%s :NOT CODED YET 3\n' % ('/'.join(cmd), element))
+                        raise e
         else:
             try:
-                m = getattr(m, element)
+                # dashes (vs underscores) cause issues in Python and other languages
+                if '-' in element:
+                    m = getattr(m, element.replace('-','_'))
+                else:
+                    m = getattr(m, element)
                 cmd.append(element)
             except AttributeError:
                 # the verb/element was not found
                 if len(cmd) == 0:
-                    exit('cli4: /%s - not found' % (element))
+                    sys.stderr.write('cli4: /%s - not found\n' % (element))
                 else:
-                    exit('cli4: /%s/%s - not found' % ('/'.join(cmd), element))
+                    sys.stderr.write('cli4: /%s/%s - not found\n' % ('/'.join(cmd), element))
+                raise e
 
     if content and params:
-        exit('cli4: /%s - content and params not allowed together' % (command))
+        sys.stderr.write('cli4: /%s - content and params not allowed together\n' % (command))
+        raise Exception
     if content:
         params = content
 
@@ -122,27 +171,27 @@ def run_command(cf, method, command, params=None, content=None, files=None):
         identifier2 = [None]
     for i2 in identifier2:
         try:
-            if method is 'GET':
+            if method == 'GET':
                 r = m.get(identifier1=identifier1,
                           identifier2=i2,
                           identifier3=identifier3,
                           params=params)
-            elif method is 'PATCH':
+            elif method == 'PATCH':
                 r = m.patch(identifier1=identifier1,
                             identifier2=i2,
                             identifier3=identifier3,
                             data=params)
-            elif method is 'POST':
+            elif method == 'POST':
                 r = m.post(identifier1=identifier1,
                            identifier2=i2,
                            identifier3=identifier3,
                            data=params, files=files)
-            elif method is 'PUT':
+            elif method == 'PUT':
                 r = m.put(identifier1=identifier1,
                           identifier2=i2,
                           identifier3=identifier3,
                           data=params)
-            elif method is 'DELETE':
+            elif method == 'DELETE':
                 r = m.delete(identifier1=identifier1,
                              identifier2=i2,
                              identifier3=identifier3,
@@ -154,11 +203,14 @@ def run_command(cf, method, command, params=None, content=None, files=None):
                 # more than one error returned by the API
                 for x in e:
                     sys.stderr.write('cli4: /%s - %d %s\n' % (command, x, x))
-            exit('cli4: /%s - %d %s' % (command, e, e))
+            sys.stderr.write('cli4: /%s - %d %s\n' % (command, e, e))
+            raise e
         except CloudFlare.exceptions.CloudFlareInternalError as e:
-            exit('cli4: InternalError: /%s - %d %s' % (command, e, e))
+            sys.stderr.write('cli4: InternalError: /%s - %d %s\n' % (command, e, e))
+            raise e
         except Exception as e:
-            exit('cli4: /%s - %s - api error' % (command, e))
+            sys.stderr.write('cli4: /%s - %s - api error\n' % (command, e))
+            raise e
 
         results.append(r)
     return results
@@ -166,6 +218,9 @@ def run_command(cf, method, command, params=None, content=None, files=None):
 def write_results(results, output):
     """dump the results"""
 
+    if output is None:
+        return
+
     if len(results) == 1:
         results = results[0]
 
@@ -175,6 +230,8 @@ def write_results(results, output):
         pass
     else:
         # anything more complex (dict, list, etc) should be dumped as JSON/YAML
+        if output is None:
+            results = None
         if output == 'json':
             try:
                 results = json.dumps(results,
@@ -191,14 +248,21 @@ def write_results(results, output):
             results = yaml.safe_dump(results)
         if output == 'ndjson':
             # NDJSON support seems like a hack. There has to be a better way
-            writer = jsonlines.Writer(sys.stdout)
-            writer.write_all(results)
-            writer.close()
+            try:
+                writer = jsonlines.Writer(sys.stdout)
+                writer.write_all(results)
+                writer.close()
+            except (BrokenPipeError, IOError):
+                pass
             return
 
-    sys.stdout.write(results)
-    if not results.endswith('\n'):
-        sys.stdout.write('\n')
+    if results:
+        try:
+            sys.stdout.write(results)
+            if not results.endswith('\n'):
+                sys.stdout.write('\n')
+        except (BrokenPipeError, IOError):
+            pass
 
 def do_it(args):
     """Cloudflare API via command line"""
@@ -207,34 +271,39 @@ def do_it(args):
     output = 'json'
     raw = False
     dump = False
+    dump_from_web = False
+    profile = None
     method = 'GET'
 
     usage = ('usage: cli4 '
              + '[-V|--version] [-h|--help] [-v|--verbose] [-q|--quiet] '
-             + '[-j|--json] [-y|--yaml] [-n|ndjson]'
+             + '[-j|--json] [-y|--yaml] [-n|ndjson] '
              + '[-r|--raw] '
              + '[-d|--dump] '
+             + '[-a|--api] '
+             + '[-p|--profile profile-name] '
              + '[--get|--patch|--post|--put|--delete] '
              + '[item=value|item=@filename|@filename ...] '
-             + '/command...')
+             + '/command ...')
 
     try:
         opts, args = getopt.getopt(args,
-                                   'VhvqjyrdGPOUD',
+                                   'Vhvqjyrdap:GPOUD',
                                    [
                                        'version',
                                        'help', 'verbose', 'quiet', 'json', 'yaml', 'ndjson',
                                        'raw',
-                                       'dump',
+                                       'dump', 'api',
+                                       'profile=',
                                        'get', 'patch', 'post', 'put', 'delete'
                                    ])
     except getopt.GetoptError:
-        exit(usage)
+        sys.exit(usage)
     for opt, arg in opts:
         if opt in ('-V', '--version'):
-            exit('Cloudflare library version: %s' % (CloudFlare.__version__))
+            sys.exit('Cloudflare library version: %s' % (CloudFlare.__version__))
         if opt in ('-h', '--help'):
-            exit(usage)
+            sys.exit(usage)
         elif opt in ('-v', '--verbose'):
             verbose = True
         elif opt in ('-q', '--quiet'):
@@ -243,16 +312,20 @@ def do_it(args):
             output = 'json'
         elif opt in ('-y', '--yaml'):
             if yaml is None:
-                exit('cli4: install yaml support')
+                sys.exit('cli4: install yaml support')
             output = 'yaml'
         elif opt in ('-n', '--ndjson'):
             if jsonlines is None:
-                exit('cli4: install jsonlines support')
+                sys.exit('cli4: install jsonlines support')
             output = 'ndjson'
         elif opt in ('-r', '--raw'):
             raw = True
+        elif opt in ('-p', '--profile'):
+            profile = arg
         elif opt in ('-d', '--dump'):
             dump = True
+        elif opt in ('-a', '--api'):
+            dump_from_web = True
         elif opt in ('-G', '--get'):
             method = 'GET'
         elif opt in ('-P', '--patch'):
@@ -265,8 +338,14 @@ def do_it(args):
             method = 'DELETE'
 
     if dump:
-        dump_commands()
-        exit(0)
+        cf = CloudFlare.CloudFlare(debug=verbose, raw=raw, profile=profile)
+        dump_commands(cf)
+        sys.exit(0)
+
+    if dump_from_web:
+        cf = CloudFlare.CloudFlare(debug=verbose, raw=raw, profile=profile)
+        dump_commands_from_web(cf)
+        sys.exit(0)
 
     digits_only = re.compile('^-?[0-9]+$')
     floats_only = re.compile('^-?[0-9.]+$')
@@ -280,8 +359,8 @@ def do_it(args):
         if arg[0] == '@':
             # a file to be uploaded - used in workers/script - only via PUT
             filename = arg[1:]
-            if method != 'PUT':
-                exit('cli4: %s - raw file upload only with PUT' % (filename))
+            if method not in ['PUT','POST']:
+                sys.exit('cli4: %s - raw file upload only with PUT or POST' % (filename))
             try:
                 if filename == '-':
                     content = sys.stdin.read()
@@ -289,7 +368,7 @@ def do_it(args):
                     with open(filename, 'r') as f:
                         content = f.read()
             except IOError:
-                exit('cli4: %s - file open failure' % (filename))
+                sys.exit('cli4: %s - file open failure' % (filename))
             continue
         tag_string, value_string = arg.split('=', 1)
         if value_string.lower() == 'true':
@@ -298,28 +377,31 @@ def do_it(args):
             value = False
         elif value_string == '' or value_string.lower() == 'none':
             value = None
-        elif value_string[0] is '=' and value_string[1:] == '':
-            exit('cli4: %s== - no number value passed' % (tag_string))
-        elif value_string[0] is '=' and digits_only.match(value_string[1:]):
+        elif value_string[0] == '=' and value_string[1:] == '':
+            sys.exit('cli4: %s== - no number value passed' % (tag_string))
+        elif value_string[0] == '=' and digits_only.match(value_string[1:]):
             value = int(value_string[1:])
-        elif value_string[0] is '=' and floats_only.match(value_string[1:]):
+        elif value_string[0] == '=' and floats_only.match(value_string[1:]):
             value = float(value_string[1:])
-        elif value_string[0] is '=':
-            exit('cli4: %s== - invalid number value passed' % (tag_string))
+        elif value_string[0] == '=':
+            sys.exit('cli4: %s== - invalid number value passed' % (tag_string))
         elif value_string[0] in '[{' and value_string[-1] in '}]':
             # a json structure - used in pagerules
             try:
                 #value = json.loads(value) - changed to yaml code to remove unicode string issues
                 if yaml is None:
-                    exit('cli4: install yaml support')
-                value = yaml.safe_load(value_string)
+                    sys.exit('cli4: install yaml support')
+                try:
+                    value = yaml.safe_load(value_string)
+                except yaml.parser.ParserError as e:
+                    raise ValueError
             except ValueError:
-                exit('cli4: %s="%s" - can\'t parse json value' % (tag_string, value_string))
-        elif value_string[0] is '@':
+                sys.exit('cli4: %s="%s" - can\'t parse json value' % (tag_string, value_string))
+        elif value_string[0] == '@':
             # a file to be uploaded - used in dns_records/import - only via POST
             filename = value_string[1:]
             if method != 'POST':
-                exit('cli4: %s=%s - file upload only with POST' % (tag_string, filename))
+                sys.exit('cli4: %s=%s - file upload only with POST' % (tag_string, filename))
             files = {}
             try:
                 if filename == '-':
@@ -327,7 +409,7 @@ def do_it(args):
                 else:
                     files[tag_string] = open(filename, 'rb')
             except IOError:
-                exit('cli4: %s=%s - file open failure' % (tag_string, filename))
+                sys.exit('cli4: %s=%s - file open failure' % (tag_string, filename))
             # no need for param code below
             continue
         else:
@@ -336,12 +418,10 @@ def do_it(args):
         if tag_string == '':
             # There's no tag; it's just an unnamed list
             if params is None:
-                params = []
-            try:
-                params.append(value)
-            except AttributeError:
-                exit('cli4: %s=%s - param error. Can\'t mix unnamed and named list' %
-                     (tag_string, value_string))
+                params = value
+            else:
+                sys.exit('cli4: %s=%s - param error. Can\'t mix unnamed and named list' %
+                         (tag_string, value_string))
         else:
             if params is None:
                 params = {}
@@ -349,21 +429,29 @@ def do_it(args):
             try:
                 params[tag] = value
             except TypeError:
-                exit('cli4: %s=%s - param error. Can\'t mix unnamed and named list' %
-                     (tag_string, value_string))
+                sys.exit('cli4: %s=%s - param error. Can\'t mix unnamed and named list' %
+                         (tag_string, value_string))
 
     # what's left is the command itself
-    if len(args) != 1:
-        exit(usage)
-    command = args[0]
+    if len(args) < 1:
+        sys.exit(usage)
+    commands = args
 
-    cf = CloudFlare.CloudFlare(debug=verbose, raw=raw)
-    results = run_command(cf, method, command, params, content, files)
-    write_results(results, output)
+    try:
+        cf = CloudFlare.CloudFlare(debug=verbose, raw=raw, profile=profile)
+    except Exception as e:
+        sys.exit(e)
+
+    for command in commands:
+        try:
+            results = run_command(cf, method, command, params, content, files)
+            write_results(results, output)
+        except Exception as e:
+            if len(commands) > 1:
+                continue
 
 def cli4(args):
     """Cloudflare API via command line"""
 
     do_it(args)
-    exit(0)
-
+    sys.exit(0)
